BEGIN
  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ACCESS_REVIEW_DETAILS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ACCESS_REVIEW_DETAILS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ACCESS_REVIEW_DETAILS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ACCESS_REVIEW_DETAILS_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'ACCESS_REVIEW_SUMMARY_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM ACCESS_REVIEW_SUMMARY_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'ACCESS_REVIEW_SUMMARY_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW ACCESS_REVIEW_SUMMARY_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REASSIGNMENTS_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REASSIGNMENTS_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REASSIGNMENTS_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REASSIGNMENTS_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEWER_ITEM_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEWER_ITEM_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEWER_ITEM_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEWER_ITEM_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'USER_PARAMETER_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM USER_PARAMETER_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'USER_PARAMETER_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW USER_PARAMETER_V';
  END LOOP;

  FOR k IN (SELECT null FROM all_synonyms WHERE owner = 'PUBLIC' and synonym_name = 'REVIEW_ITEM_ACTION_COMBINED_V') LOOP
    EXECUTE IMMEDIATE 'DROP PUBLIC SYNONYM REVIEW_ITEM_ACTION_COMBINED_V';
  END LOOP;
   FOR i IN (SELECT null FROM all_views WHERE owner = user and view_name = 'REVIEW_ITEM_ACTION_COMBINED_V') LOOP
    EXECUTE IMMEDIATE 'DROP VIEW REVIEW_ITEM_ACTION_COMBINED_V';
  END LOOP;
END;
/

CREATE VIEW access_review_details_v AS
   SELECT DISTINCT
    item.id review_item_id,
    item.last_override_action_id last_override_action,
    instance.id review_instance_id,
    instance.status review_status,
    instance.start_date,
    instance.end_date,
    instance.valid_to_date,
    action.last_update,
    item.approved_time,
    instance.item_count,
    multi_owner.review_owners,
    multi_reviewer.reviewer_ids reviewer_id,
      CASE
        WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN TO_CHAR(final_reviewer.display_name)
        ELSE TO_CHAR(multi_reviewer.reviewers)
      END reviewer_display_name,
      CASE
        WHEN item.final_cert_by_unique_user_id IS NOT NULL THEN TO_CHAR(substr(final_reviewer.dn, 4, instr(final_reviewer.dn, ',', 1, 1)-4))
        ELSE TO_CHAR(multi_reviewer.user_cn)
      END reviewer_cn,
    reassigned_by_history.reviewers reassigned_by_user_disp_name,
    reassigned_from_history.reviewers reassigned_from_user_disp_name,
    reassigned_by_history.reassigned_date,
    reassigned_by_history.reasons reassigned_reasons,
    reassigned_by_history.rev_comments reassigned_comments,
    defn.name review_name,
    TO_CHAR(substr(defn.description,0,4000)) description,
    defn.review_type,
    defn.next_schedule_date,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN shacc.suser
        ELSE usr.id
      END user_id,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN CAST(shacc.display_name as nvarchar2(2000))
        ELSE CAST(usr.display_name as nvarchar2(2000))
      END user_display_name,
      CASE
        WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN TO_CHAR(shacc.user_cn)
        ELSE TO_CHAR(substr(usr.dn, 4, instr(usr.dn, ',', 1, 1)-4))
      END user_cn,
    action.action reviewer_action,
    reason.name modify_reason_name,
    reason.id modify_reason_id,
      CASE
        WHEN fulfillment.review_item_id IS NOT NULL THEN 1
        ELSE 0
      END remove_completed,
    usr.unique_user_id,
    action.review_order,
    action.reviewer_comment,
    perm.id permission_id,
    perm.name permission_name,
    acct.id account_id,
    acct.account_id account_native_id,
    acct.name account_name,
    acct.relation_to_user_type account_relation,
    TO_CHAR(app.id) application_id,
    TO_CHAR(app_ds.name) application_name,
    item.final_decision,
    item.final_decision_time,
    role.id role_id,
    role.name role_name,
    item.item_type,
    brole.name brole_name,
    brole.id business_role_id,
    action.fulfillment_instructions,
    esc_reviewer.display_name esc_reviewer,
    sup.unique_user_id supervisor_id,
    sup.display_name supervisor_display_name
  FROM review_item item
    JOIN review_instance instance ON item.review_instance_id = instance.id AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING')
    JOIN (SELECT
      review_instance.id inst_id,
      defn.id defn_id,
      CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
      CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END AS description,
      CASE WHEN staged_defn.next_schedule_date IS NULL THEN defn.next_schedule_date ELSE staged_defn.next_schedule_date END AS next_schedule_date,
      defn.review_type,
      defn.deleted
      FROM review_definition defn
      JOIN review_instance ON defn.id = review_instance.review_definition
      LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON instance.id = defn.inst_id
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN suser sup ON sup.unique_user_id = item.unique_supervisor_id AND sup.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN application app ON app.id =
      CASE
        WHEN perm.id IS NULL THEN acct.application
        ELSE perm.application
      END
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    LEFT JOIN data_source app_ds ON app_ds.id = app.id
    LEFT JOIN review_item_action action ON action.review_item_id = item.id AND action.action <> 'REASSIGNED'
    LEFT JOIN review_item_action_reason reason ON reason.id = action.reason_id
    LEFT JOIN reviewer_instance inst2 ON instance.id = inst2.review_instance_id AND inst2.type = 'RO'
    LEFT JOIN reviewer_instance inst3 ON instance.id = inst3.review_instance_id
    LEFT JOIN ( SELECT DISTINCT
      item.id item_id,
      LISTAGG((TO_CHAR(multi_reviewers.id)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewer_ids,
      LISTAGG((TO_CHAR(multi_reviewers.unique_user_id)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewer_unique_ids,
      LISTAGG((TO_CHAR(multi_reviewers.display_name)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) reviewers,
      LISTAGG((TO_CHAR(substr(multi_reviewers.dn, 4, instr(multi_reviewers.dn, ',', 1, 1)-4))), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY multi_reviewers.display_name) user_cn
      FROM review_item item
      LEFT JOIN (SELECT DISTINCT reviewer_instance_id, unique_user_id FROM reviewer_user_instance) rrui ON rrui.reviewer_instance_id = item.curr_reviewer_instance_id
      LEFT JOIN suser multi_reviewers ON rrui.unique_user_id = multi_reviewers.unique_user_id AND multi_reviewers.effective = 1
      GROUP BY item.id) multi_reviewer ON multi_reviewer.item_id = item.id
    LEFT JOIN suser final_reviewer ON item.final_cert_by_unique_user_id = final_reviewer.unique_user_id AND final_reviewer.effective = 1
    LEFT JOIN suser acting_reviewer ON action.acted_by_user_unique_id = acting_reviewer.unique_user_id AND acting_reviewer.effective = 1
    LEFT JOIN (SELECT
      ria.review_item_id,
      LISTAGG(TO_CHAR(usr.display_name), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reviewers
      FROM  review_item_action ria
      LEFT JOIN reviewer_instance ri ON ri.id = ria.reviewer_instance_id
      LEFT JOIN suser usr ON ri.single_reviewer_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_from_history ON reassigned_from_history.review_item_id = item.id
    LEFT JOIN (SELECT
      ria.review_item_id,
      LISTAGG(TO_CHAR(usr.display_name), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reviewers,
      LISTAGG(COALESCE(TO_CHAR(reason.name), 'NULL_REASON'), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reasons,
      LISTAGG(TO_CHAR(CAST(ria.assigned AS DATE), 'YYYY-MM-DD HH24:MI:SS'), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY ria.assigned) AS reassigned_date,
      LISTAGG(TO_CHAR(COALESCE (NULLIF(TO_CHAR(ria.reviewer_comment), ''), 'NULL_COMMENT')), ' | ' on overflow truncate) WITHIN GROUP (ORDER BY ria.assigned) AS rev_comments
      FROM  review_item_action ria
      LEFT JOIN review_item_action_reason reason ON ria.reason_id = reason.id
      LEFT JOIN suser usr ON ria.reassigned_by_user_unique_id = usr.unique_user_id AND usr.effective = 1
      WHERE ria.action = 'REASSIGNED'
      GROUP BY ria.review_item_id) reassigned_by_history ON reassigned_by_history.review_item_id = item.id
    LEFT JOIN shared_accounts_v shacc ON shacc.unique_account_id = acct.unique_account_id
    LEFT JOIN ( SELECT DISTINCT
      defn_1.name review_name,
      ri.id rid,
      LISTAGG((TO_CHAR(s.display_name)), ' | ' on overflow truncate) WITHIN GROUP(ORDER BY s.display_name) review_owners
      FROM review_instance ri
      JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
      LEFT JOIN reviewer_instance rev_owner ON ri.id = rev_owner.review_instance_id AND rev_owner.type = 'RO'
      LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner.id
      LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective = 1
      GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = item.review_instance_id
    LEFT JOIN (SELECT
      esc_reviewers.review_instance_id,
      LISTAGG(TO_CHAR(esc_reviewers.display_name), ' | ' on overflow truncate) WITHIN GROUP (ORDER BY esc_reviewers.display_name) display_name
      FROM
        (SELECT DISTINCT
        TO_CHAR(u.display_name) display_name,
        rri.review_instance_id
        FROM review_item ri
        JOIN reviewer_instance rri ON ri.review_instance_id = rri.review_instance_id AND rri.type = 'EO'
        JOIN reviewer_user_instance rriu ON rriu.reviewer_instance_id = rri.id
        JOIN suser u ON u.unique_user_id = rriu.unique_user_id AND u.effective = 1) esc_reviewers
      GROUP BY esc_reviewers.review_instance_id) esc_reviewer ON esc_reviewer.review_instance_id = item.review_instance_id
    LEFT JOIN ( SELECT
      review_item_id,
      action,
      change_item_status
      FROM change_request_item cri
      LEFT JOIN change_item_action cia ON cri.latest_action = cia.id
      WHERE cri.change_request_type LIKE 'REMOVE' || '%' AND action = 'FULFILLED' AND (change_item_status = 'VERIFIED' OR change_item_status = 'COMPLETED')) fulfillment ON item.id = fulfillment.review_item_id

  WHERE
    defn.deleted = 0
    AND (acting_reviewer.unique_user_id LIKE '%' || multi_reviewer.reviewer_unique_ids || '%' OR acting_reviewer.unique_user_id IS NULL
      AND (action.review_order =
      ( SELECT DISTINCT min(act.review_order) min
        FROM review_item_action act
        WHERE (act.action = 'PENDING' OR act.action = 'ESCALATE') AND act.review_item_id = item.id)
      OR action.acted_by_policy = 1)
      OR acting_reviewer.unique_user_id IS NOT NULL AND action.shared_queue = 1)
    AND (action.action <> 'PENDING' OR item.final_decision IS NULL);


CREATE VIEW access_review_summary_v AS 
 SELECT 
    instance.id instance_id,
    defn.name,
    defn.description,
    defn.review_type,
    defn.next_schedule_date,
    defn.unique_review_def_id,
    multi_owner.review_owners review_inst_owner_display_name,
    instance.status,
    instance.status_comment,
    instance.start_date,
    instance.expected_end_date,
    instance.end_date,
    instance.valid_to_date,
    instance.canceled_by,
    instance.task_count,
    instance.task_complete_count,
    CASE
      WHEN instance.task_complete_count = 0 THEN 0
      ELSE round((instance.task_complete_count / instance.task_count) * 100, 1)
    END AS percent_complete,
    canceler.display_name canceled_by_name,
    instance.item_count,
    instance.item_complete_count,
    instances.reviewer_count,
    (SELECT decisions.decision_count
      FROM (SELECT inst.id  instance,
          it.final_decision  decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'ASSIGN_USER')  assign_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) AS decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'KEEP') AS keep_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision = 'REMOVE')  remove_count,
    (SELECT decisions.decision_count
      FROM ( SELECT inst.id AS instance,
          it.final_decision decision,
          COUNT(it.final_decision) decision_count
        FROM review_instance inst
          LEFT JOIN review_item it ON it.review_instance_id = inst.id
        GROUP BY inst.id, it.final_decision) decisions
      WHERE decisions.instance = instance.id AND decisions.decision IN ('MODIFY_ENTITY', 'MODIFY_ACCESS', 'MODIFY_SUPERVISOR'))  modify_count
  FROM review_instance instance
    JOIN (SELECT
        review_instance.id inst_id,
        defn.id defn_id,
        CASE WHEN staged_defn.name IS NULL THEN defn.name ELSE staged_defn.name END AS name,
        CASE WHEN staged_defn.description IS NULL THEN defn.description ELSE staged_defn.description END description,
        defn.review_type,
        defn.next_schedule_date,
        defn.deleted,
        defn.unique_review_def_id
      FROM review_definition defn
        JOIN review_instance ON defn.id = review_instance.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = review_instance.staged_review_definition_id) defn ON defn.inst_id = instance.id
    JOIN (SELECT inst.id AS instance,
          COUNT(DISTINCT rui.unique_user_id)  reviewer_count
         FROM review_instance inst
           LEFT JOIN reviewer_instance ri ON ri.review_instance_id = inst.id
           LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = ri.id
           WHERE ri.task_count > 0
        GROUP BY inst.id) instances ON instances.instance = instance.id
    LEFT JOIN suser canceler ON instance.canceled_by = canceler.unique_user_id AND canceler.effective = 1
    LEFT JOIN ( SELECT DISTINCT 
        defn_1.name review_name,
        ri.id AS rid,
        LISTAGG((TO_CHAR(s.display_name)), ' | ') WITHIN GROUP (ORDER BY s.display_name) AS review_owners
        FROM review_instance ri
          JOIN review_definition defn_1 ON ri.review_definition = defn_1.id
          LEFT JOIN reviewer_instance rev_owner_1 ON ri.id = rev_owner_1.review_instance_id AND rev_owner_1.type = 'RO'
          LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = rev_owner_1.id
          LEFT JOIN suser s ON rui.unique_user_id = s.unique_user_id AND s.effective=1
        GROUP BY defn_1.name, ri.id) multi_owner ON multi_owner.rid = instance.id
  WHERE defn.deleted = 0 AND instance.status NOT IN ('PREVIEW', 'PREVIEW_CANCELED', 'PREVIEW_STARTING');

CREATE VIEW reassignments_v AS
SELECT
    history.review_instance_id,
    history.review_item_id,
    history.review_order,
    review.name AS review_name,
    review.status AS review_status,
    review.start_date,
    review.end_date,
      CASE
        WHEN item.item_type = 'PERMISSION' THEN perm.name
        WHEN item.item_type = 'ACCOUNT' THEN acct.name
        WHEN item.item_type = 'ROLE' THEN role.name
        WHEN item.item_type = 'BUSINESS_ROLE' THEN brole.name
        WHEN (item.item_type = 'DIRECT_REPORT' OR item.item_type = 'USER') THEN suser.display_name
      END AS review_item,
    item.item_type,
    history.reassigned_by_user_unique_id,
    history.reassigned_by,
    history.reassigned_from,
    history.reassigned_to,
    reason.id AS reason_id,
    reason.name AS reassigned_reasons,
    history.reviewer_comment,
    history.reassignment_date
  FROM ( SELECT
      ria_1.id,
      ria_1.review_item_id,
      ria_1.review_order,
      ria_1.reason_id,
      ria_1.reviewer_comment,
      ria_1.action,
      ria_1.reassigned_by_user_unique_id,
        CASE
          WHEN reassign_from_multi.users IS NOT NULL AND reassign_from_multi.grps IS NOT NULL THEN concat(reassign_from_multi.users, concat(' | ', reassign_from_multi.grps))
          ELSE COALESCE(reassign_from_multi.grps, reassign_from_multi.users, TO_CHAR(reassign_from.display_name))
        END AS reassigned_from,
      reassign_by.display_name AS reassigned_by,
      reassign_to.display_name AS reassigned_to,
      ria_1.review_instance_id,
        CASE
          WHEN ria_1.assigned IS NULL AND ria_1.action = 'REASSIGNED' THEN null_date.assigned
          ELSE ria_1.assigned
        END AS reassignment_date
    FROM review_item_action ria_1
      JOIN reviewer_instance ri ON ri.id = ria_1.reviewer_instance_id
      JOIN suser reassign_from ON ri.single_reviewer_unique_id = reassign_from.unique_user_id AND reassign_from.effective = 1
      JOIN suser reassign_by ON reassign_by.unique_user_id = ria_1.reassigned_by_user_unique_id AND reassign_by.effective = 1
      JOIN suser reassign_to ON reassign_to.unique_user_id = ria_1.reassigned_to_user_unique_id AND reassign_to.effective = 1
      LEFT JOIN review_item_action null_date ON null_date.review_item_id = ria_1.review_item_id AND null_date.review_order = ria_1.review_order AND null_date.action = 'PENDING'
      LEFT JOIN ( SELECT 
          ria_1_1.id,
          LISTAGG(TO_CHAR(reassign_from_users_1.display_name),' | ') WITHIN GROUP (ORDER BY reassign_from_users_1.display_name) users,
          LISTAGG(TO_CHAR(reassign_from_grps_1.name),' | ') WITHIN GROUP (ORDER BY reassign_from_grps_1.name) grps
        FROM review_item_action ria_1_1
          JOIN ( SELECT DISTINCT reviewer_set_instance.reviewer_instance_id,
                            reviewer_set_instance.unique_user_id,
                            reviewer_set_instance.unique_group_id
                           FROM reviewer_set_instance) rsi_1 ON ria_1_1.reviewer_instance_id = rsi_1.reviewer_instance_id
          LEFT JOIN suser reassign_from_users_1 ON rsi_1.unique_user_id = reassign_from_users_1.unique_user_id AND reassign_from_users_1.effective = 1
          LEFT JOIN sgroup reassign_from_grps_1 ON reassign_from_grps_1.unique_group_id = rsi_1.unique_group_id AND reassign_from_grps_1.effective = 1
        WHERE ria_1_1.action = 'REASSIGNED'
        GROUP BY ria_1_1.id) reassign_from_multi ON reassign_from_multi.id = ria_1.id
    WHERE ria_1.action = 'REASSIGNED') history
    LEFT JOIN review_item_action_reason reason ON history.reason_id = reason.id
    JOIN review_item item ON item.id = history.review_item_id
    LEFT JOIN suser ON item.unique_user_id = suser.unique_user_id AND suser.effective = 1
    LEFT JOIN spermission perm ON perm.unique_permission_id = item.unique_permission_id AND perm.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN business_role brole ON brole.id = item.business_role_id
    LEFT JOIN role_policy role ON role.id = item.role_id
    JOIN ( SELECT 
        ri.id AS inst_id,
        defn.id AS defn_id,
        COALESCE(staged_defn.name, defn.name) AS name,
        ri.status,
        ri.start_date,
        ri.end_date
      FROM review_definition defn
        JOIN review_instance ri ON defn.id = ri.review_definition
        LEFT JOIN review_definition staged_defn ON staged_defn.id = ri.staged_review_definition_id
      WHERE ri.status NOT IN ('PREVIEW_CANCELED')) review ON review.inst_id = history.review_instance_id;

CREATE VIEW reviewer_item_v AS
  SELECT item.id AS item_id,
        CASE
            WHEN rui.unique_user_id IS NULL THEN item.final_cert_by_unique_user_id
            ELSE rui.unique_user_id
        END AS unique_user_id,
      defn.review_type
  FROM review_item item
     LEFT JOIN reviewer_user_instance rui ON rui.reviewer_instance_id = item.curr_reviewer_instance_id AND item.final_cert_by_unique_user_id IS NULL
     JOIN review_instance instance ON item.review_instance_id = instance.id
     JOIN review_definition defn ON instance.review_definition = defn.id;

CREATE VIEW user_parameter_v AS 
  SELECT DISTINCT 
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.id IS NULL THEN user_acct.id
      ELSE usr.id
    END AS user_id,
    CASE
      WHEN item.item_type = 'ACCOUNT' AND usr.display_name IS NULL THEN user_acct.display_name
      ELSE usr.display_name
    END AS user_display_name
  FROM review_item item
    LEFT JOIN suser usr ON usr.unique_user_id = item.unique_user_id AND usr.effective = 1
    LEFT JOIN saccount acct ON acct.unique_account_id = item.unique_account_id AND acct.effective = 1
    LEFT JOIN saccount_user acct_user ON acct_user.saccount = acct.id
    LEFT JOIN suser user_acct ON user_acct.id = acct_user.suser;

CREATE VIEW review_item_action_combined_v AS
 SELECT ria.id ,
       ria.id AS review_item_action,
       ri.id review_item,
       ria.review_instance_id,
       ria.review_item_status,
       ria.assigned,
       ria.last_update,
       ria.reviewer_comment,
       ria.action AS individual_action,
       rialast.id AS last_action,
       riaover.id AS last_override_action,
       riausers.display_name AS supervisor_display_name,
       usr.id AS user_id,
       usr.display_name AS user_display_name,
       usr.unique_user_id AS user_unique_id,
       usr.department AS user_department,
       usr.location AS user_location,
       usr.job_title AS user_job_title,
       usr.job_code AS user_job_code,
       usr.employee_type AS user_employee_type,
       usr.employee_status AS user_employee_status,
       usr.idm_tree_name AS user_idm_tree_name,       
       usr.risk AS user_risk,
       usr.display_description AS user_display_description,
       null AS user_is_supervisor,
       usr.workforce_id AS user_workforce_id,
       perm.id  AS permission_id,
       perm.name AS permission_name,
       acct.id AS account_id,
       acct.account_id AS account_external_id,
       acct.name  AS account_name,
       app.id AS application_id,
       ds.name AS application_name,
       app.show_acct_name_on_review_item AS show_account_name,
       app.show_permission_hierachy,
       ri.item_type,
       ri.relation_to_user_type,
       ri.parent_review_item_id,
       rolepol.id AS role_id,
       rolepol.name  AS role_name,
       brole.id AS business_role_id,
       brole.name AS business_role_name,
       CASE
           WHEN brole.id IS NOT NULL THEN brole.name
           WHEN rolepol.id IS NOT NULL THEN rolepol.name
           WHEN perm.id IS NOT NULL THEN perm.name
           WHEN acct.id IS NOT NULL THEN acct.name
           ELSE NULL
       END AS id_name,
       rrui.unique_user_id,
       ri.authorized_by_policy,
       su.display_name,
       usr.deleted AS user_deleted,
       ri.auto_approved AS approved,
       CASE
         WHEN ri.auto_approved = 1 THEN 'AUTO_APPROVED'
         WHEN ri.auto_approved = 0 THEN 'USER_APPROVED'
         ELSE 'NOT_APPROVED'
       END as approved_so,
       CASE
           WHEN brole.id IS NOT NULL AND brole.approval_state = 'ARCHIVE'THEN 1
           WHEN rolepol.id IS NOT NULL THEN rolepol.deleted
           WHEN perm.id IS NOT NULL THEN perm.deleted
           WHEN acct.id IS NOT NULL THEN acct.deleted
           ELSE 0
       END AS item_deleted,
      modifier.name AS name,
      attr.attr_string_val,
      attrdef.attribute_key,
      attr_1.attr_string_val AS changed_from,
      attrdef_1.attribute_key AS changed_from_key
FROM review_item_action ria
INNER JOIN review_item ri ON (ri.id = ria.review_item_id)
INNER JOIN review_instance r ON (r.id = ri.review_instance_id)
INNER JOIN reviewer_user_instance rrui ON (rrui.reviewer_instance_id = ria.reviewer_instance_id)
LEFT JOIN review_item_attr_value attr ON (attr.modify_action_id = ri.id AND ria.review_item_status = 'CERTIFIED' AND attr.change_type = 1)
LEFT JOIN attribute_definition attrdef ON (attrdef.id = attr.attribute_definition_id)
LEFT JOIN review_item_attr_value attr_1 ON (attr_1.modify_action_id = ria.id AND ria.review_item_status = 'CERTIFIED' AND attr_1.change_type = 2 AND attr_1.attribute_definition_id = attr.attribute_definition_id)
LEFT JOIN attribute_definition attrdef_1 ON (attrdef_1.id = attr_1.attribute_definition_id) 
LEFT JOIN suser usr ON (usr.unique_user_id = ri.unique_user_id AND usr.effective = 1)
LEFT JOIN suser su ON (su.unique_user_id = ria.acted_by_user_unique_id AND su.deleted = 0 AND su.effective = 1)
LEFT JOIN review_item_action rialast ON (rialast.id = ri.last_action_id)
LEFT JOIN review_item_action riaover ON (riaover.id = ri.last_override_action_id)
LEFT JOIN spermission perm ON (perm.unique_permission_id = ri.unique_permission_id
                                  AND perm.effective = 1)
LEFT JOIN saccount acct ON (acct.unique_account_id = ri.unique_account_id
                             AND acct.effective = 1)
LEFT JOIN application app ON (app.id = CASE
                                  WHEN perm.id IS NULL THEN acct.application
                                  ELSE perm.application
                              END)
LEFT JOIN data_source ds ON (ds.id = app.id)
LEFT JOIN role_policy rolepol ON (rolepol.id = ri.role_id)
LEFT JOIN business_role brole ON (brole.id = ri.business_role_id)
LEFT JOIN ( SELECT 
  ria_1.review_item_id,
  LISTAGG(TO_CHAR(su_1.display_name), ' | ') WITHIN GROUP(ORDER BY su_1.display_name) name
  FROM review_item_action ria_1
  JOIN reviewer_user_instance rrui_1 ON rrui_1.reviewer_instance_id = ria_1.reviewer_instance_id
  JOIN suser su_1 ON su_1.unique_user_id = rrui_1.unique_user_id AND su_1.deleted = 0 AND su_1.effective = 1
  WHERE ria_1.review_item_status = 'REASSIGNED'
  GROUP BY ria_1.review_item_id) modifier ON modifier.review_item_id = ria.review_item_id
LEFT JOIN ( SELECT 
  suser.display_name,
  usi.review_item_action,
  usi.updated_supervisor_ids
  FROM ( SELECT 
          ria.review_item_action, 
          substr(ria.assigned_user_unique_ids, 4, 32) AS updated_supervisor_ids
          FROM review_item_action_users ria) usi
  JOIN suser on suser.unique_user_id = usi.updated_supervisor_ids AND suser.deleted = 0 AND suser.effective = 1) riausers ON riausers.review_item_action = ria.id;


GRANT SELECT ON igops.access_review_details_v TO ig_report_role;
CREATE PUBLIC SYNONYM access_review_details_v FOR igops.access_review_details_v;

GRANT SELECT ON igops.access_review_summary_v TO ig_report_role;
CREATE PUBLIC SYNONYM access_review_summary_v FOR igops.access_review_summary_v;

GRANT SELECT ON igops.reassignments_v TO ig_report_role;
CREATE PUBLIC SYNONYM reassignments_v FOR igops.reassignments_v;

GRANT SELECT ON igops.reviewer_item_v TO ig_report_role;
CREATE PUBLIC SYNONYM reviewer_item_v FOR igops.reviewer_item_v;

GRANT SELECT ON igops.user_parameter_v TO ig_report_role;
CREATE PUBLIC SYNONYM user_parameter_v FOR igops.user_parameter_v;

GRANT SELECT ON igops.review_item_action_combined_v TO ig_report_role;
CREATE PUBLIC SYNONYM review_item_action_combined_v FOR igops.review_item_action_combined_v;